# @author: Matt Owen
# @date:   6/2/2010
# bury.zkeleton.R
# stores a zkeleton object as a set of files

# @z:    zkeletor object
# @file: file
bury.zkeleton <- function(z, file = NULL, path=".", ...) {
  # output files
  cat("* creating files in working directory = ")
  message(getwd())


  # file lists
  files <- c(# zelig hooks
             "zelig2", "zelig3",
             # simulation related
             "setx", "sim", "sim.setx", "qi",
             # printing related
             "summary.sim", "print.summary.sim", "describe"
             )


  #
  for (fi in files) {
    # annoying code to handle that zelig2 and zelig3 don't have dots
    sep <- ifelse(fi == "zelig2" || fi == "zelig3", "", ".")
    full <- paste(path, z$model, "R",
                  paste(fi, paste(z$model, "R", sep="."), sep=sep),
                  sep="/")

    # display file name being written
    cat(" * writing file: ")
    message(full)

    # open file
    f <- file(full, open="w")

    credit <- how.to <- canned <- ""

    # get credit text
    if (is.logical(z$credit) && z$credit)
      credit <- replace.zkeleton(gettext(paste(fi, "credit")),
                                 author=z$author,
                                 date=z$date,
                                 model=z$model,
                                 model.function=z$model.function                               
                                 )

    # get how-to text
    if (is.logical(z$help) && z$help)
      how.to <- replace.zkeleton(gettext(paste(fi, "how-to")),
                                 author=z$author,
                                 date=z$date,
                                 model=z$model,
                                 model.function=z$model.function                               
                                 )
    
    # get canned data and make string replacements
    canned <- replace.zkeleton(gettext(paste(fi, "canned")),
                               author=z$author,
                               date=z$date,
                               model=z$model,
                               model.function=z$model.function
                               )

    # beautify code text
    canned <- zlean(canned)

    # debug
    # write file
    writeLines(c(credit, "\n", how.to, "\n", canned), f, sep="")
    close(f)
  }
  

  # return invisibly
  invisible(z)
}


replace.zkeleton <- function(mes, author, model, model.function, date)
{
  # error-checking
  if (length(grep("(<<|>>)", c(author, model, model.function))))
    warning("Warning: potentially invalid parameters passed to sub. Please make sure << or >> exist in author, model, or model.function")

  # make replacements
  mes <- gsub("<<author>>", author, mes)
  mes <- gsub("<<model function>>", model.function, mes)
  mes <- gsub("<<model name>>", model, mes)
  mes <- gsub("<<model>>", model, mes)
  mes <- gsub("<<date>>", date, mes)
  mes <- gsub("<<year>>", format(Sys.time(), "%Y"), mes)

  # return
  mes
}
