# @author: Matt Owen
# @date:   6/16/2010
# info: generates "DESCRIPTION" file
#       for package

# generic
description <- function(x, ...) UseMethod("description")

# 
description.zkeleton <- function (z, ..., verbose=T) {
  # email
  email <- z$email
  if (!is.null(email))
    email <- paste(z$author,
                   paste("<", email, ">", sep="")
                   )

  # maintainer
  # gives the format:
  #  author, author, author <email, email, email>
  maintainer <- ifelse(is.null(email),
                       z$author,
                       paste(head(z$author, 1), email, collapse=", ")
                       )
  
  # actual list-object
  s <- list(Package = z$model,
            Title   = z$model,
            Version = "1.0",
            Date    = gsub("/", "-", z$date),
            Author  = z$author,
            Maintainer = maintainer,
            Depends    = paste(c("R (>= 2.8.1)", "Zelig", z$depends), sep=", ", collapse=", "),
            Description = "",
            License = "GPL (>=2)",
            LazyLoad= "yes"
            )

  # initialize an empty vector
  lines <- c()
  
  # append to vector

  for (key in names(s)) {
    line <- paste(key, s[[key]], sep=": ")
    lines <- c(lines, line)
  }

  # return
  lines
}

# 
write.description <- function(z, ..., verbose=T) {
  # open description file
  fi <- file(paste(z$path, z$model, "DESCRIPTION", sep="/"), open="w")

  # write description file
  writeLines(description(z), fi)

  # close file handle
  close(fi)
}
