# @author: Matt Owen
# @date:   6/1/2010
# @file:   qi.mprobit.R
# @info:   Generates quantities of interest for
#          Kosuke Imai's Multinomial Probit Model


# @obj:    
# @simpar: 
# @x:      
# @x1:     
# @y:      
# return:  
qi.mprobit <- function(obj, simpar=NULL, x, x1=NULL, y=NULL) {
  # simulate values for mnp model
  p <- predict(obj, newdata=x$newdata)
  ev.p <- p$p
  pr.p <- p$o[,,1]


  # simulate more values, preferably
  # from a different data-set
  if (!is.null(x1)) {
    q <- predict(obj, newdata=x1$newdata)
    ev.q <- q$p
    pr.q <- q$o[,,1]
  }

  # init
  qi.stats <- list()
  qi.name <- list()


  # list of the avg & quantiles
  qi.stats$quants <- c()
  qi.name$quants <- "Expected Values: E(Y|X)"

  # compute expected values and certainty levels 
  if (length(ev.p) && !is.na(ev.p))
    for (k in 1:ncol(ev.p)) {
      lis <- c(# mean of data set
               mean(ev.p[,k]),
                              
               # standard deviation
               sd(ev.p[,k]),
                              
               # quantiles of data set
               quantile(ev.p[,k],
                        probs=c(.025, .5, .975),
                        names=F
                        )
               )
      qi.stats$quants <- rbind(qi.stats$quants, lis)
    }

  # compute modes and likely ordering of preferences
  qi.stats$modes <- c()
  qi.name$modes <- "Predicted Values: Y|X"
  if (length(pr.p) && !is.na(pr.p)) {
    for (k in 1:ncol(pr.p))
      qi.stats$modes <- c(qi.stats$modes, mode.zelig(pr.p[,k]))

    for (k in length(qi.stats$modes)) {
      qi.stats$predicted[[k]] <- sum(as.numeric(
                                                pr.p ==
                                                qi.stats$modes[[k]]))/length(pr.p)
      
    }
  }

  # a second setx object was passed in, so
  # we'll try and compute the first-difference

  qi.stats$fds <- c()
  qi.name$fds <- "E(Y|X1) - E(Y|X): First Differences of Choices"

  if (!is.null(x1) &&
      length(ev.p) && !is.na(ev.p) &&
      length(ev.q) && !is.na(ev.q)) {

    for (k in 1:ncol(ev.p)) {
      fd <- ev.p[,k] - ev.q[,k]
      lis <- c(# mean of data set
               mean(fd),

               # standard deviation
               sd(fd),

               # quantiles of data set
               quantile(fd,
                        probs=c(.025, .5, .975),
                        names=F
                        )
               )
      qi.stats$fds <- rbind(qi.stats$fds, lis)
    }
  }

  
  # make an array, so we can allocate all the space now
  colsize <- ncol(pr.p)
  qi.stats$counts <- array(dim=c(colsize, colsize))

  # name rows/cols
  colnames(qi.stats$counts) <- colnames(pr.p)
  rownames(qi.stats$counts) <- 1:colsize

  # number of matches for k in specified column
  for (key in colnames(pr.p))
    for (k in 1:colsize)
      qi.stats$counts[k, key] <- sum(pr.p[,key] == k)

  # normalize columns
  qi.stats$counts <- qi.stats$counts/length(pr.p)*colsize

  # name the columns/rows
  names(qi.stats$modes) <- colnames(pr.p)

  if (length(dim(qi.stats$fds)) == 2) {
    rownames(qi.stats$fds) <- colnames(ev.p)
    colnames(qi.stats$fds) <- c("mean", "sd", "2.5%", "50%", "97.5%")
  }

  if (length(dim(qi.stats$quants)) == 2) {
    rownames(qi.stats$quants) <- colnames(ev.p)
    colnames(qi.stats$quants) <- c("mean", "sd", "2.5%", "50%", "97.5%")
  }
  list(qi.stats=qi.stats,
       qi.name =qi.name
       )
}
