# @author: Matt Owen
# @date:   6/17/2010
# info:

write.Rd <- function(name, aliases=c(), title="", doctype=NULL,
                     ...,
                     author="",
                     keywords=NULL,
                     path = "."
                     ) {
  # open file handle
  f <- file(paste(path, "man", paste(name, "Rd", sep="."), sep="/"), open="w")

  # write file
  writeLines(paste(c(wrap.item("name", name, sep=""),
                  wrap.item("alias", aliases, sep="")
                  ),
                collapse="\n"
             ), f)

  # doc-type
  if (!is.null(doctype))
    writeLines(wrap.item("docType", doctype, sep=""), f)

  # title
  writeLines(wrap.item("title", title, sep=""), f)

  mc <- tail(as.list(match.call()), -1)
  mc$name <- mc$aliases <- mc$title <- NULL
  mc$path <- mc$doctype <- mc$keywords <- mc$author <- NULL
  
  for (key in names(mc)) {
    val <- mc[[key]]
    writeLines(wrap.item(key, val), f)
  }
  if (!is.null(author))
    writeLines(wrap.item("author", paste(author, collapse=", ")),
               f
               )
  writeLines(paste(c(wrap.item("keyword", keywords,sep=" ")),
                collapse="\n"),
             f
             )

  # close handle
  close(f)
}

# @tag:   character string specifying the tag
#         that is being written
# @text:  character vector containing values for tags
# return: formatted character string
#         \"tag" {<sep>"text"<sep>}
wrap.item <- function(tag, text, sep="\n") {
  res <- NULL
  for (t in text) {
    res <- c(res,
             paste(paste("\\", tag, sep=""),
                   paste("{", t, "}", sep=sep),
                   sep="")
             )
  }
  res
}
