# @author: Matt Owen
# @date:   6/2/2010


# dummy functions
dummy <- function(x, ...) UseMethod("dummy")
dummy.zelig <- function(z, ...) "dummy function"
dummy.zkeleton <- dummy.zelig


# thanks function
thanks <- function(x) UseMethod("thanks")
thanks.zkeleton <- function(z, ...) "thanks for using zelig!"


# @model:         character-string specifying the model name
# @superuser:     if T, 
# @documentation: add in-file documentation for new users
zkeleton <- function(model = NULL,
                     model.function = "",
                     depends=NULL,
                     help=T,
                     documentation = T,
                     credit=T,
                     mode = "w",
                     lang = "en",
                     author = "Mysterious, Nameless Author",
                     email = "an.email@somewhere.net",
                     path = ".",
                     force = T,
                     write.files = T
                     ) {
  # determine whether we are running interactively


  
  # error-catching
  # let program break, but do it with error
  # messages
  if (!is.character(model))
    stop("Error: model must be a character-string")

  if (!is.logical(c(help, documentation)))
    stop("Error: help and documentation must be logical")

  if (!(mode %in% c("w", "w-", "w+", "r", "p")))
    stop("Error: unknown file mode")

  safe.expr <- "^[a-zA-Z][a-zA-Z0-9\\._]*$"

  if (regexpr(safe.expr, model) != 1)
    stop("error: model is not in a valid format")

  if (regexpr(safe.expr, model.function) != 1)
    stop("error: model.function is not in a valid format")

  if (any(regexpr(safe.expr, depends) != 1))
    stop("error: a dependency is not in a valid format")
  


  # overwrite files that already exist?
  overwrite <- mode == "w" || mode == "wt"

  # create a zkeleton type object
  # named after the adorable he-man villain
  zkeletor <- list(model = model,
                   model.function=model.function,
                   depends=depends,
                   overwrite = mode == "w+",

                   # publication/citation info
                   author = author,
                   date = format(Sys.time(), "%D"),
                   email = email,

                   # file io
                   path = path,
                   mode = mode,

                   # file commentary
                   documentation = documentation,
                   help = help,
                   credit = credit,
                   interactive=interactive()
                   )

  # 
  class(zkeletor) <- "zkeleton"

  z <- zkeletor

  # skeleton
  package.skeleton(zkeletor$model,
                   "dummy.zkeleton",
                   force = force, #zkeletor$overwrite,
                   path = path,
                   namespace = T
                   )

  # important namespace material
  add.lines <- c("sim, ",
                 "sim, setx.",
                 "setx, ",
                 "summary, sim.",
                 "print, summary.sim."
                 )
  add.lines <- paste(add.lines, model, sep="")
  add.lines <- paste("S3method(", add.lines, ")", sep="")


  # open file stream
  title.namespace <- paste(path, model, "NAMESPACE", sep="/")
  namespace <- file(title.namespace, "a")



  if (write.files) {
    # output
    message()
    message("writing from template files")

    # things just aren't the same, zkeleton   
    bury(zkeletor, path=path)

    # message about editing namespace
    cat("* appending to namespace: ")
    message(title.namespace)

    # editing NAMESPACE
    writeLines(add.lines, namespace)

    # edit DESCRIPTION
    write.description(zkeletor)

    # write help for dummy.zkeleton
    write.Rd(name="dummy.zkeleton", aliases="dummy.zkeleton",
             title="function that does not do much",
             description="A space-filling function.  Serves no real purpose",
             details="none",
             value="a character-string containing an unimportant message",
             author=author,
             path=paste(path, model, sep="/")
             )

    # write help for the entire package
    name <- paste(z$model, "package", sep="-")
    write.Rd(name=name,
             aliases=name,
             doctype="package",
             title="a model implemented in Zelig",        
             description="a zelig module, automatically generated by zkeleton",
             details="some-package.R is a Zelig-implementation of the BLANK software package",
             author=author,
             keywords=c("zelig"),
             path=paste(path, model, sep="/")
             )
  }

  close(namespace)

  invisible(zkeletor)
}
