\name{zkeleton}

\alias{zkeleton}
\alias{skeleton.zelig}

\title{
zkeleton.R
}

\description{
zkeleton assists in the creation of zelig modules
}

\usage{
zkeleton(model, model.function, ...)
}

\arguments{
  \item{model}{
    a character-string naming the package
  }
  \item{model.function}{
    a character-string specifying the function to be called at run-time by zelig
  }
  \item{depends}{
    a vector of character-strings specifying the packages on which this
    new model depends
  }
  \item{help}{
    a boolean value specifying whether helpful commentary on how-to
    build a zelig function should be included in the source code
  }
  \item{mode}{
    a character string of "w", "w+", or "a" that specifies whether
    zkeleton will write, overwrise, or append to existing files in the
    directory path
  }
  \item{author}{
    a vector of character-strings specifying the authors of the package
  }
  \item{email}{
    a vector of character-strings specifying the emails of the authors
    of the package
  }
  \item{path}{
    a character-string specifying the path of the model to be
    constructed
  }
  \item{force}{
    a boolean value to be passed to the function "package.skeleton"
  }
  \item{write.files}{
    a boolean specifying whether the files should actually be written,
    or whether the "zkeleton" object should merely be returned (for use
    with print, summary, etc.)
  }
}

\details{
  The zkeleton package is used to rapidly develop zelig modules that
  adhere to new strict specifications of zelig modules.  Thus, it is a
  useful tool to write code consistent with the myriad of other zelig
  modules.
}

\value{
  a zkeleton object.  that is, a list containing information about the
  path, author name, email address, etc. of the newly created package
}

\author{
  Matthew Owen
}

\note{
  zkeleton will handle most any component of a zelig module.  The
  exception to this rule is the qi.<<model name>> function.  This must
  be almost entirely user specified, as it is impossible to in general
  determine which quantities are actually of interest.
}

\seealso{
  package.skeleton, Zelig, zelig
}
\examples{
# include zkeleton library
library(zkeleton)

# create zelig module
zkeletor <- zkeleton(model="multiprobit",
                     model.function="mnp",
                     depends="MNP",
                     author="your name",
		     email="your.name@wherever.biz"
		     )
# view results
summary(zkeletor)

# FINALLY: modify multiprobit/R/qi.multiprobit.R with
#          an appropriate qi function
}
\keyword{ skeleton }
\keyword{ zelig }
\keyword{ wizard }
